package com.mortals.xhx.common.utils;

import cn.hutool.core.util.HexUtil;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.util.encoders.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.security.Security;

/**
 * @Description 平台请求天府健康通接口demo
 * demo 依赖于 hutool-all 5.5.7 以及 bcprov-jdk15to18 1.64
 * @ClassName RioDemo
 * @Author ScarletDrop
 * @Version 1.0
 **/
public class RioUtil {
    static {
        Security.addProvider(new BouncyCastleProvider());
    }


    /**
     * 加密
     *
     * @param content 内容
     * @param sm4     sm4密钥
     * @return 加密字符串
     */
    public static String encryptHex(String content, String sm4) {
        String inputHex = HexUtil.encodeHexStr(content.getBytes());
        byte[] encryptRet = new byte[0];
        try {
            SecretKeySpec sm4Key = new SecretKeySpec(HexUtil.decodeHex(sm4.toCharArray()), "SM4");
            Cipher cipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", BouncyCastleProvider.PROVIDER_NAME);
            cipher.init(Cipher.ENCRYPT_MODE, sm4Key);
            encryptRet = cipher.doFinal(HexUtil.decodeHex(inputHex.toCharArray()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Base64.toBase64String(encryptRet);
    }


    /**
     * 解密
     *
     * @param content 内容
     * @param sm4     sm4密钥
     * @return 解密字符串
     */
    public static String decrypt(String content, String sm4) {
        byte[] decoded = Base64.decode(content);
        byte[] encryptRet = new byte[0];
        try {
            SecretKeySpec sm4Key = new SecretKeySpec(HexUtil.decodeHex(sm4.toCharArray()), "SM4");
            Cipher cipher = Cipher.getInstance("SM4/ECB/PKCS7Padding", BouncyCastleProvider.PROVIDER_NAME);
            cipher.init(Cipher.DECRYPT_MODE, sm4Key);
            encryptRet = cipher.doFinal(decoded);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new String(encryptRet, Charset.forName("UTF-8"));
    }

//    public static void main(String[] args) {
//       String a =  encryptHex("scdsjzx","b4a8af568f71136a6b1f8e8a4c9ebc2d");
//       log.info("a===={}",a);
//        String B =  decrypt(a,"b4a8af568f71136a6b1f8e8a4c9ebc2d");
//        log.info("B===={}",B);
//    }
}
