package com.mortals.xhx.module.app.service.impl;

import cn.hutool.core.net.url.UrlBuilder;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import com.mortals.framework.model.DbMatchMode;
import com.google.common.collect.Maps;

import cn.hutool.core.io.FileUtil;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.pdu.app.AppPdu;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.common.utils.ZipUtils;
import com.mortals.xhx.feign.app.device.IAppFeign;
import com.mortals.xhx.module.app.model.*;
import com.mortals.xhx.module.app.service.*;
import com.mortals.xhx.module.site.model.SiteEntity;
import com.mortals.xhx.module.site.service.SiteService;
import com.mortals.xhx.utils.EncodeUtil;
import com.mortals.xhx.version.model.VersionEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.app.dao.AppDao;
import org.springframework.util.ObjectUtils;

import java.io.File;
import java.nio.charset.Charset;
import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.CUSTAPP_ROOT_PATH;

/**
 * AppService
 * 自助终端应用 service实现
 *
 * @author zxfei
 * @date 2022-11-28
 */
@Service("appService")
@Slf4j
public class AppServiceImpl extends AbstractCRUDServiceImpl<AppDao, AppEntity, Long> implements AppService {

    @Autowired
    private UploadService uploadService;
    @Autowired
    private SiteService siteService;
    @Autowired
    private AppDatasetService appDatasetService;
    @Autowired
    private AppInfoFieldService appInfoFieldService;
    @Autowired
    private AppInfoTempleteFieldService appInfoTempleteFieldService;
    @Autowired
    private AppVersionService appVersionService;
//    @Autowired
//    private IAppFeign appFeign;

    @Override
    protected void findAfter(AppEntity params, PageInfo pageInfo, Context context, List<AppEntity> list) throws AppException {
        //排序
        if (!ObjectUtils.isEmpty(params.getIdList())) {
            log.info("请求的排序id列表:{}", JSON.toJSONString(params.getIdList()));
            try {
                //去除idlist中不存在的
                Set<Long> idSet = list.stream().map(item -> item.getId()).collect(Collectors.toSet());
                List<Long> idList = params.getIdList();
                Iterator<Long> ite = idList.iterator();
                while (ite.hasNext()) {
                    Long next = ite.next();
                    if (!idSet.contains(next)) {
                        ite.remove();
                    }
                }
                //根据idList进行排序
                for (int i = 0; i < idList.size(); i++) {
                    Long id = idList.get(i);
                    for (int j = 0; j < list.size(); j++) {
                        if (id.equals(list.get(j).getId())) {
                            //判断位置是否一直
                            if (i != j) {
                                //交换
                                Collections.swap(list, i, j);
                            }
                        }
                    }
                }
            } catch (Exception e) {
                log.error("排序异常！", e);
            }
        }
        //统计站点
        list.forEach(item -> {
            List<AppEntity> appEntityList = this.find(new AppQuery().appCode(item.getAppCode()), context);
            item.setApplianceSiteScope(appEntityList.size());
            item.setSiteIdList(appEntityList.stream().map(AppEntity::getSiteId).collect(Collectors.toList()));
            //构建访问地址
            SiteEntity siteEntity = siteService.getCache(item.getSiteId().toString());
            String domainUrl = GlobalSysInfo.getParamValue(Constant.PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11078");
            if (!ObjectUtils.isEmpty(siteEntity)) {
                //请求地址 http://domian/app/siteCode/appcode/html
                item.setCustUrl(UrlBuilder.of(domainUrl)
                        .addPath(CUSTAPP_ROOT_PATH)
                        .addPath(siteEntity.getSiteCode())
                        .addPath(item.getAppCode())
                        .addPath(item.getVersion().toString())
                        .toString());
            } else {
                item.setCustUrl("");
            }
            //构建图标地址
            item.setAppIconUrl(UrlBuilder.of(domainUrl)
                    .addPath(item.getAppIconPath()).build());
        });
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    public Rest<String> appDistribute(AppEntity appEntity, Context context) {
        SiteEntity siteCache = siteService.getCache(appEntity.getSiteId().toString());
        //判断文件是否存在
        String targetFilePath = uploadService.getFilePath(appEntity.getFilePath());
        if (FileUtil.isEmpty(new File(targetFilePath))) {
            throw new AppException("部署文件不存在！");
        }

        if (!FileUtil.getSuffix(appEntity.getFilePath()).equals("zip")) {
            throw new AppException("部署只支持zip文件！");
        }
        //部署路径是否存在 如果不存在 创建目录，
        //文件构成目录  基础path +站点编码+应用编码  /basepath/sitecode/appcode
        String path = GlobalSysInfo.getParamValue(Constant.APP_BASE_DISTRIBUTE_PATH, "/home/publish/app/homeUrl");
        String code = siteCache.getSiteCode();
        String appcode = appEntity.getAppCode();
        String disPath = path + File.separator + code + File.separator + appcode;
        //判断目标目录是否存在 如果存在 这删除
        if (FileUtil.isDirectory(disPath)) {
            FileUtil.del(disPath);
        }
        //ZipUtils.unzip(new File(targetFilePath), disPath);
        String fileEncode = "UTF-8";
        try {
            fileEncode = EncodeUtil.getEncode(targetFilePath, true);
        } catch (Exception e) {
            log.error("异常", e);
        }
        ZipUtil.unzip(targetFilePath, disPath, Charset.forName(fileEncode));
        //更新
        appEntity.setDistributeFilePath(disPath);
        appEntity.setDistribute(YesNoEnum.YES.getValue());
        appEntity.setUpdateTime(new Date());
        this.update(appEntity, context);
        return Rest.ok("应用部署成功！");
    }

    @Override
    public Rest<Void> appCommonDistribute(AppEntity appEntity, Context context) {
        //判断文件是否存在
        String targetFilePath = uploadService.getFilePath(appEntity.getFilePath());
        if (FileUtil.isEmpty(new File(targetFilePath))) {
            throw new AppException("部署文件不存在！");
        }
        if (!FileUtil.getSuffix(appEntity.getFilePath()).equals("zip")) {
            throw new AppException("部署只支持zip文件！");
        }
        //部署路径是否存在 如果不存在 创建目录，
        //文件构成目录 /home/publish/app/common
        String disPath = "/home/publish/app/common";

        //判断目标目录是否存在 如果存在 这删除
        if (FileUtil.isDirectory(disPath)) {
            FileUtil.del(disPath);
        }
        // ZipUtils.unzip(new File(targetFilePath), disPath);
        String fileEncode = "UTF-8";
        try {
            fileEncode = EncodeUtil.getEncode(targetFilePath, true);
        } catch (Exception e) {
            log.error("异常", e);
        }
        ZipUtil.unzip(targetFilePath, disPath, Charset.forName(fileEncode));

        return Rest.ok("基础应用部署成功！");
    }

    @Override
    public Rest<String> cloneAppsBySites(List<AppEntity> appList, List<SiteEntity> siteList, Context context) {
        log.info("克隆应用开始!");
        //查询被克隆的站点是否已经存在当前克隆的应用 如果存在 则跳过
        for (AppEntity appEntity : appList) {
            for (SiteEntity siteEntity : siteList) {
                AppQuery appQuery = new AppQuery();
                appQuery.setAppCode(appEntity.getAppCode());
                appQuery.setSiteId(siteEntity.getId());
                int count = this.count(appQuery, context);
                if (count == 0) {
                    //不存在，可以克隆当前应用到当前站点
                    AppEntity appClone = new AppEntity();
                    appClone.initAttrValue();
                    BeanUtils.copyProperties(appEntity, appClone, BeanUtil.getNullPropertyNames(appEntity));
                    appClone.setVersion(1);
                    appClone.setDistribute(YesNoEnum.NO.getValue());
                    appClone.setDownDevCount(0);
                    appClone.setSiteId(siteEntity.getId());
                    appClone.setSiteName(siteEntity.getSiteName());
                    appClone.setDistributeFilePath("");
                    appClone.setShelves(YesNoEnum.NO.getValue());
                    appClone = this.save(appClone, context);
                    //保存数据集
                    List<AppDatasetEntity> appDatasetList = appClone.getAppDatasetList();
                    if (!ObjectUtils.isEmpty(appDatasetList)) {
                        for (AppDatasetEntity appDatasetEntity : appDatasetList) {
                            AppDatasetEntity appDatasetClone = new AppDatasetEntity();
                            appDatasetClone.initAttrValue();
                            BeanUtils.copyProperties(appDatasetEntity, appDatasetClone, BeanUtil.getNullPropertyNames(appDatasetEntity));
                            appDatasetClone.setAppId(appClone.getId());
                            appDatasetService.save(appDatasetClone, context);

                            //查询属性列表
                            List<AppInfoFieldEntity> appInfoFieldList = appInfoFieldService.find(new AppInfoFieldQuery().datasetId(appDatasetEntity.getId()));
                            //保存属性列表
                            //List<AppInfoFieldEntity> appInfoFieldList = appDatasetEntity.getAppInfoFieldList();
                            if (!ObjectUtils.isEmpty(appInfoFieldList)) {
                                ArrayList<AppInfoFieldEntity> cloneList = new ArrayList<>();
                                for (AppInfoFieldEntity appInfoFieldEntity : appInfoFieldList) {
                                    AppInfoFieldEntity appInfoFieldClone = new AppInfoFieldEntity();
                                    appInfoFieldClone.initAttrValue();
                                    BeanUtils.copyProperties(appInfoFieldEntity, appInfoFieldClone, BeanUtil.getNullPropertyNames(appInfoFieldEntity));
                                    appInfoFieldClone.setDatasetId(appDatasetClone.getId());
                                    cloneList.add(appInfoFieldClone);
                                }
                                if (!ObjectUtils.isEmpty(cloneList)) {
                                    appInfoFieldService.save(cloneList, context);
                                }
                            }
                        }
                    }

                    //克隆属性模板
                    List<AppInfoTempleteFieldEntity> appInfoTempleteFieldEntities = appInfoTempleteFieldService.find(new AppInfoTempleteFieldQuery().appId(appEntity.getId()));
                    if (!ObjectUtils.isEmpty(appInfoTempleteFieldEntities)) {
                        ArrayList<AppInfoTempleteFieldEntity> appInfoTempleteFieldCloneList = new ArrayList<>();
                        for (AppInfoTempleteFieldEntity appInfoTempleteFieldEntity : appInfoTempleteFieldEntities) {
                            AppInfoTempleteFieldEntity appInfoTempleteFieldClone = new AppInfoTempleteFieldEntity();
                            appInfoTempleteFieldClone.initAttrValue();
                            BeanUtils.copyProperties(appInfoTempleteFieldEntity, appInfoTempleteFieldClone, BeanUtil.getNullPropertyNames(appInfoTempleteFieldEntity));
                            appInfoTempleteFieldClone.setAppId(appClone.getId());
                            appInfoTempleteFieldCloneList.add(appInfoTempleteFieldClone);
                        }
                        if (!ObjectUtils.isEmpty(appInfoTempleteFieldCloneList)) {
                            appInfoTempleteFieldService.save(appInfoTempleteFieldCloneList, context);
                        }
                    }
                }
            }
        }
        return Rest.ok("克隆应用完成!");
    }


    @Override
    public AppDao getAppDao() {
        return this.dao;
    }


    @Override
    protected void validData(AppEntity entity, Context context) throws AppException {
        if (ObjectUtils.isEmpty(entity.getAppCode())) {
            throw new AppException("应用编码不能为空！");
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(AppEntity entity, Context context) throws AppException {
        //校验appCode是否唯一
        Long siteId = entity.getSiteId();
        String appCode = entity.getAppCode();
        int count = this.count(new AppQuery().siteId(siteId).appCode(appCode), context);
        if (count > 0) {
            throw new AppException("当前站点已存在应用编码");
        }
        super.saveBefore(entity, context);
    }

    @Override
    protected void saveAfter(AppEntity entity, Context context) throws AppException {
        //添加版本
        AppVersionEntity versionEntity = new AppVersionEntity();
        versionEntity.initAttrValue();
        versionEntity.setAppId(entity.getId());
        versionEntity.setAppName(entity.getAppName());
        versionEntity.setVersion(entity.getVersion());
        versionEntity.setNotes("".equals(entity.getNotes()) ? "应用初始创建！" : entity.getNotes());
        versionEntity.setFileName(entity.getFileName());
        versionEntity.setFilePath(entity.getFilePath());
        versionEntity.setDistributeFilePath(entity.getDistributeFilePath());
        versionEntity.setCreateTime(new Date());
        versionEntity.setCreateUserId(this.getContextUserId(context));
        appVersionService.save(versionEntity, context);
        super.saveAfter(entity, context);
    }

    @Override
    public AppEntity update(AppEntity entity, Context context) throws AppException {
        this.updateBefore(entity, context);
        String filePath = entity.getFilePath();
        String fileName = entity.getFileName();
        entity.setFilePath(null);
        entity.setFileName(null);

        //判断如果应用下架，通知自助服务终端
//        if(entity.getShelves()==YesNoEnum.NO.getValue()){
//            AppPdu appPdu = new AppPdu();
//            appPdu.setAppId(entity.getId());
//            appFeign.forbidden(appPdu);
//        }
        int iRet = this.dao.update(entity);
        if (iRet == 0) {
            throw new AppException(-1002, "更新失败！");
        } else {
            entity.setFileName(fileName);
            entity.setFilePath(filePath);
            this.updateAfter(entity, context);
            return entity;
        }

    }

    @Override
    protected void updateAfter(AppEntity entity, Context context) throws AppException {

        AppVersionQuery appVersionQuery = new AppVersionQuery();
        appVersionQuery.setAppId(entity.getId());
        List<OrderCol> orderColList = new ArrayList<>();
        orderColList.add(new OrderCol("version", OrderCol.DESCENDING));
        appVersionQuery.setOrderColList(orderColList);
        List<AppVersionEntity> appVersionEntities = appVersionService.find(appVersionQuery, context);
        //获取最大版本号
        if (!ObjectUtils.isEmpty(appVersionEntities)) {
            AppVersionEntity curAppVersionEntity = appVersionEntities.get(0);
            //  String versionNum = StrUtil.subAfter(curAppVersionEntity.getVersion(), Constant.VERSION_PREFIX, false);
            Integer versionNum = curAppVersionEntity.getVersion();
            // Integer newVersionNum = versionNum++;
            AppVersionEntity appVersionEntity = new AppVersionEntity();
            appVersionEntity.initAttrValue();
            AppVersionEntity versionEntity = new AppVersionEntity();
            versionEntity.initAttrValue();
            versionEntity.setAppId(entity.getId());
            versionEntity.setAppName(entity.getAppName());
            versionEntity.setVersion(++versionNum);
            versionEntity.setNotes(entity.getNotes());
            versionEntity.setFileName(entity.getFileName());
            versionEntity.setFilePath(entity.getFilePath());
            versionEntity.setDistributeFilePath(entity.getDistributeFilePath());
            versionEntity.setCreateTime(new Date());
            versionEntity.setCreateUserId(this.getContextUserId(context));
            appVersionService.save(versionEntity, context);

        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        //删除模板属性
        List<AppInfoTempleteFieldEntity> appInfoTempleteFieldDeleteList = appInfoTempleteFieldService.find(new AppInfoTempleteFieldQuery().appIdList(Arrays.asList(ids)));
        if (!ObjectUtils.isEmpty(appInfoTempleteFieldDeleteList)) {
            Long[] idList = appInfoTempleteFieldDeleteList.stream().map(item -> item.getId()).toArray(Long[]::new);
            appInfoTempleteFieldService.remove(idList, context);
        }
        //删除版本信息
        List<AppVersionEntity> appVersionDeleteList = appVersionService.find(new AppVersionQuery().appIdList(Arrays.asList(ids)));
        if (!ObjectUtils.isEmpty(appVersionDeleteList)) {
            Long[] idList = appVersionDeleteList.stream().map(item -> item.getId()).toArray(Long[]::new);
            appVersionService.remove(idList, context);
        }

        //删除数据集
        List<AppDatasetEntity> appDatasetDeleteList = appDatasetService.find(new AppDatasetQuery().appIdList(Arrays.asList(ids)));
        if (!ObjectUtils.isEmpty(appDatasetDeleteList)) {
            Long[] idLists = appDatasetDeleteList.stream().map(item -> item.getId()).toArray(Long[]::new);
            appDatasetService.remove(idLists, context);
            List<AppInfoFieldEntity> appInfoFieldDeleteList = appInfoFieldService.find(new AppInfoFieldQuery().datasetIdList(appDatasetDeleteList.stream().map(AppDatasetEntity::getAppId).collect(Collectors.toList())));
            if (!ObjectUtils.isEmpty(appInfoFieldDeleteList)) {
                Long[] idList = appInfoFieldDeleteList.stream().map(item -> item.getId()).toArray(Long[]::new);
                appInfoFieldService.remove(idList, context);
            }
        }

        super.removeAfter(ids, context, result);
    }

    public static void main(String[] args) {

    }
}