package com.mortals.xhx.module.single.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.single.dao.SingleMatterFileDao;
import com.mortals.xhx.module.single.model.SingleMatterFileEntity;
import com.mortals.xhx.module.single.model.SingleMatterFileQuery;
import com.mortals.xhx.module.single.service.SingleMatterFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
* SingleMatterFileService
* 单事项申请文件 service实现
*
* @author zxfei
* @date 2023-06-07
*/
@Service("singleMatterFileService")
@Slf4j
public class SingleMatterFileServiceImpl extends AbstractCRUDServiceImpl<SingleMatterFileDao, SingleMatterFileEntity, Long> implements SingleMatterFileService {

    @Autowired
    private MatterService matterService;

    @Override
    protected void validData(SingleMatterFileEntity entity, Context context) throws AppException {
        if(entity.getMatterId()==null){
            throw new AppException("基础事项id不能为空");
        }
        if(StringUtils.isEmpty(entity.getFilename())){
            throw new AppException("文件名称不能为空");
        }
        if(StringUtils.isEmpty(entity.getFileUrl())){
            throw new AppException("上传文件不能为空");
        }
    }

    @Override
    public SingleMatterFileEntity saveFile(SingleMatterFileEntity entity, Context context) throws AppException {
        this.validData(entity,context);
        SingleMatterFileEntity temp = this.selectOne(new SingleMatterFileQuery().matterId(entity.getMatterId()));
        if(temp!=null){
            if(!temp.getFileUrl().equals(entity.getFileUrl())){
                SingleMatterFileEntity update = new SingleMatterFileEntity();
                update.setId(temp.getId());
                update.setFilename(entity.getFilename());
                update.setFileUrl(entity.getFileUrl());
                update.setUpdateTime(new Date());
                if(context!=null && context.getUser()!=null){
                    entity.setUpdateUserId(context.getUser().getId());
                }
                dao.update(update);
            }
        }else {
            MatterEntity matterEntity = matterService.get(entity.getMatterId());
            if(matterEntity==null){
                throw new AppException("基础事项id不正确,查不到对应的事项");
            }
            entity.setMatterCode(matterEntity.getMatterNo());
            entity.setMatterName(matterEntity.getMatterName());
            if(entity.getSiteId()==null) {
                entity.setSiteId(matterEntity.getSiteId());
            }
            if(context!=null && context.getUser()!=null){
                entity.setCreateUserId(context.getUser().getId());
            }
            entity.setCreateTime(new Date());
            dao.insert(entity);
        }
        return entity;
    }
}